<?php

/**
 * @file Workflow-ng integration for the node module 
 */

/** 
 * Implementation of hook_event_info()
 */
function node_event_info() {
  return array(
    'node_insert' => array(
      '#label' => t('Content has been created'),
      '#module' => t('Node'),
      '#arguments' => workflow_ng_events_node_arguments(t('created content'), t('content\'s author')),
      '#redirect' => TRUE,
    ),
    'node_update' => array(
      '#label' => t('Content has been updated'),
      '#module' => t('Node'),
      '#arguments' => workflow_ng_events_node_arguments(t('updated content'), t('content author')) + array(
        'node_unchanged' => array('#entity' => 'node', '#label' => t('unchanged content'), '#handler' => 'workflow_ng_events_argument_node_unchanged'),
        'author_unchanged' => array('#entity' => 'user', '#label' => t('unchanged content\'s author'), '#handler' => 'workflow_ng_events_argument_unchanged_node_author'),
      ),
      '#redirect' => TRUE,
    ),
    'node_submit' => array(
      '#label' => t('Content is going to be saved'),
      '#module' => t('Node'),
      '#arguments' => array(
        'node' => array('#entity' => 'node', '#label' => t('content'), '#saved' => TRUE),
        'author' => array('#entity' => 'user', '#label' => t('content author'), '#handler' => 'workflow_ng_events_argument_node_author'),
        'node_unchanged' => array('#entity' => 'node', '#label' => t('unchanged content'), '#handler' => 'workflow_ng_events_argument_node_unchanged'),
        'author_unchanged' => array('#entity' => 'user', '#label' => t('unchanged content\'s author'), '#handler' => 'workflow_ng_events_argument_unchanged_node_author'),
      ) + workflow_ng_events_global_user_argument(),
      '#redirect' => TRUE,
    ),
    'node_view' => array(
      '#label' => t('Content is going to be viewed'),
      '#module' => t('Node'),
      '#description' => t('Note that if drupal\'s page cache is enabled, this event won\'t be generated for pages served from cache.'),
      '#arguments' => workflow_ng_events_node_arguments(t('viewed content'), t('content author')),
    ),
    'node_delete' => array(
      '#label' => t('Content has been deleted'),
      '#module' => t('Node'),
      '#arguments' => workflow_ng_events_node_arguments(t('deleted content'), t('content author')),
      '#redirect' => TRUE,
    ),
  );
}

/*
 * Returns some arguments suitable for using it with a node
 */
function workflow_ng_events_node_arguments($node_label, $author_label) {
  return array(
    'node' => array('#entity' => 'node', '#label' => $node_label),
    'author' => array('#entity' => 'user', '#label' => $author_label, '#handler' => 'workflow_ng_events_argument_node_author'),
  ) + workflow_ng_events_global_user_argument();
}

/*
 * Implementation of hook_nodeapi
 */
function workflow_ng_nodeapi(&$node, $op, $teaser = NULL, $page = NULL) {

  if (in_array($op, array('view', 'insert', 'update', 'submit', 'delete'))) {
    //we pass the node by reference so that changes are saved automatically on op submit
    workflow_ng_invoke_event('node_'. $op, array('node' => &$node));
  }
}

/*
 * Gets the author's account of a node
 */
function workflow_ng_events_argument_node_author($node) {
  return user_load(array('uid' => $node->uid));
}

/*
 * Gets the node object, that doesn't contain the modified changes
 */
function workflow_ng_events_argument_node_unchanged($node) {
  return $node->nid ? node_load($node->nid) : $node;
}

/*
 * Gets the author of the unchanged node object
 */
function workflow_ng_events_argument_unchanged_node_author($node) {
  return workflow_ng_events_argument_node_author(workflow_ng_events_argument_node_unchanged($node));
}





/*
 * Implementation of hook_condition_info()
 */
function node_condition_info() {
  return array(
    'workflow_ng_condition_content_is_type' => array(
      '#label' => t('Content has type'),
      '#arguments' => array(
        'node' => array('#entity' => 'node', '#label' => t('Content')),
      ),
      '#description' => t('Evaluates to TRUE, if the given content has one of the selected content types.'),
      '#module' => t('Node'),
    ),
  );
}

/*
 * Condition: Check for selected content types
 */
function workflow_ng_condition_content_is_type($node, $settings) {
  return in_array($node->type, $settings['type']);
}

/*
 *  Check for content types - Configuration form
 */
function workflow_ng_condition_content_is_type_form($settings = array()) {
  $form['type'] = array(
    '#type' => 'select',
    '#title' => t('Content types'),
    '#options' => node_get_types('names'),
    '#multiple' => TRUE,
    '#default_value' => isset($settings['type']) ? $settings['type'] : array(),
    '#required' => TRUE,
  );
  return $form;
}

function workflow_ng_condition_content_is_type_submit($form_id, $form_values) {
  return array('type' => $form_values['type']);
}







/*
 * Implementation of hook_action_info()
 */
function node_action_info() {
  return array(
    'workflow_ng_action_node_set_author' => array(
      '#label' => t('Set the content author'),
      '#arguments' => array(
        'node' => array('#entity' => 'node', '#label' => t('Content')),
        'author' => array('#entity' => 'user', '#label' => t('User, which is set as author')),
      ),
      '#module' => t('Node'),
    ),
    'workflow_ng_action_node_publish' => array(
      '#label' => t('Publish/unpublish content'),
      '#arguments' => array(
        'node' => array('#entity' => 'node', '#label' => t('Content')),
      ),
      '#module' => t('Node'),
    ),
    'workflow_ng_action_node_promote' => array(
      '#label' => t('Promote/demote content to/from the front page'),
      '#arguments' => array(
        'node' => array('#entity' => 'node', '#label' => t('Content')),
      ),
      '#module' => t('Node'),
    ),
  );
}


/*
 * Modifies a node as configured
 */
function workflow_ng_action_node_set_author($node, $author) {
  $node->uid = $author->uid;
  $node->name = $author->name;
  return array('node' => $node);
}

/*
 * Sets a node's Published flag
 */
function workflow_ng_action_node_publish($node, $settings, &$arguments, &$log) {
  if ($node->status != $settings['published']) {
    $node->status = $settings['published'];
    return array('node' => $node);
  }
}

/*
 * Action node publish configuration form
 */
function workflow_ng_action_node_publish_form($settings = array(), $argument_info) {
  $form = array();
  $form['published'] = array(
    '#type' => 'checkbox',
    '#title' => t('Published'),
    '#default_value' => $settings['published'],
  );
  return $form;
}

function workflow_ng_action_node_publish_submit($form_id, $form_values) {
  return array('published' => $form_values['published']);
}

/*
 * Sets a node's Promoted flag
 */
function workflow_ng_action_node_promote($node, $settings, &$arguments, &$log) {
  if ($node->promote != $settings['promote']) {
    $node->promote = $settings['promote'];
    return array('node' => $node);
  }
}

/*
 * Action node promote configuration form
 */
function workflow_ng_action_node_promote_form($settings = array(), $argument_info) {
  $form = array();
  $form['promote'] = array(
    '#type' => 'checkbox',
    '#title' => t('Promoted to front page'),
    '#default_value' => $settings['promote'],
  );
  return $form;
}

function workflow_ng_action_node_promote_submit($form_id, $form_values) {
  return array('promote' => $form_values['promote']);
}
