<?php

/**
 * @file Workflow-ng integration for the system module
 */

/**
 * Implementation of hook_event_info()
 */
function system_event_info() {
  return array(
    'init' => array(
      '#label' => t('User is going to view a page'),
      '#module' => t('System'),
      '#description' => t('Note that if drupal\'s page cache is enabled, this event won\'t be generated for pages served from cache.'),
      '#arguments' => workflow_ng_events_global_user_argument(),
    ),
  );
}

/*
 * Implementation of hook_menu
 * As system_menu exists already, we implement workflow_ng_menu
 */
function workflow_ng_menu($may_cache) {
  if (!$may_cache) {
    workflow_ng_invoke_event('init');
  }
}


/*
 * Implementation of hook_condition_info()
 */
function system_condition_info() {
  return array(
    'workflow_ng_condition_token_compare' => array(
      '#label' => t('Textual comparison'),
      //we are not working with defined arguments, instead we make use of all available arguments
      '#arguments' => array(),
      '#description' => t('This condition just compares two texts. It returns TRUE, if both texts are equal.').
        ' '. t('You may use token replacements in both texts, e.g. the texts %text1 and %text2 are only equal, if the argument called "node" is of the type page.', array('%text1' => "[node:type]", '%text2' => 'page')),
      '#module' => t('System'),
    ),
    'workflow_ng_condition_token_numeric' => array(
      '#label' => t('Numeric comparison'),
      '#arguments' => array(),
      '#description' => t('This does a numeric comparison between two tokens. Select greater than, less than or equal to.'),
      '#module' => t('System'),
    ),
  );
}

/*
 * Condition Implementation: Token enabled comparison
 */
function workflow_ng_condition_token_compare($settings, &$arguments, &$log) {
  extract( workflow_ng_token_replace_all(array('text1', 'text2'), $settings, $arguments, $log) );
  return $settings['regex'] ? ereg($text2, $text1) : $text1 == $text2;
}

/*
 * Condition: Token enabled comparison configuration form
 */
function workflow_ng_condition_token_compare_form($settings = array(), $argument_info) {
  $form = array();
  $form['text1'] = array(
    '#type' => 'textfield',
    '#title' => 'Two texts to compare',
    '#default_value' => isset($settings['text1']) ? $settings['text1'] : '',
  );
  $form['text2'] = array(
    '#type' => 'textfield',
    '#title' => '',
    '#default_value' => isset($settings['text2']) ? $settings['text2'] : '',
  );
  $form['regex'] = array(
    '#type' => 'checkbox',
    '#title' => t('Evaluate text2 as a regular expression.'),
    '#default_value' => isset($settings['regex']) ? $settings['regex'] : 0,
  );
  workflow_ng_token_replacement_help($form, $argument_info);
  return $form;
}

function workflow_ng_condition_token_compare_submit($form_id, $form_values) {
  //returns the needed settings
  $settings = workflow_ng_token_get_settings(array('text1', 'text2'), $form_values);
  return $settings + array('regex' => $form_values['regex']);
}

/*
 * Condition implementation: token enabled numeric comparison
 */
function workflow_ng_condition_token_numeric($settings, &$arguments, &$log) {
  extract( workflow_ng_token_replace_all(array('text1', 'text2'), $settings, $arguments, $log) );
  $text1 = (float)$text1;
  $text2 = (float)$text2;

  switch ($settings['operation']) {
    case 'greater':
      $result = $text1 > $text2;
      break;
    case 'equal':
      $result = $text1 == $text2;
      break;
    case 'less':
      $result = $text1 < $text2;
      break;
  }
  return $result;
}

/*
 * Condition: token enabled numeric comparison configuration form
 */
function workflow_ng_condition_token_numeric_form($settings = array(), $argument_info) {
  $form = array();
  $form['text1'] = array(
    '#type' => 'textfield',
    '#title' => t('Text to compare'),
    '#default_value' => isset($settings['text1']) ? $settings['text1'] : '',
  );
  $form['operation'] = array(
    '#type' => 'select',
    '#title' => t('Operation'),
    '#options' => array('greater' => t('Greater than'), 'equal' => t('Equal to'), 'less' => t('Less than')),
    '#default_value' => $settings['operation'],
  );
  $form['text2'] = array(
    '#type' => 'textfield',
    '#title' => '',
    '#default_value' => isset($settings['text2']) ? $settings['text2'] : '',
  );
  workflow_ng_token_replacement_help($form, $argument_info);
  return $form;
}

function workflow_ng_condition_token_numeric_submit($form_id, $form_values) {
  $settings = workflow_ng_token_get_settings(array('text1', 'text2'), $form_values);
  return $settings + array('operation' => $form_values['operation']);
}







/*
 * Implementation of hook_action_info()
 */
function system_action_info() {
  return array(
    'workflow_ng_action_mail_to_user' => array(
      '#label' => t('Send a mail to a user'),
      '#arguments' => array(
        'user' => array('#entity' => 'user', '#label' => t('Recipient')),
      ),
      '#module' => t('System'),
      '#description' => t('You may use the token replacements for the mail\'s message body as well as the mail\'s subject.'),
    ),
    'workflow_ng_action_mail_to_users_of_role' => array(
      '#label' => t('Send mail to all users of a role'),
      '#arguments' => array(),
      '#module' => t('System'),
      '#description' => t('You may use the token replacements for the mail\'s message body as well as the mail\'s subject.'),
    ),
    'workflow_ng_action_mail' => array(
      '#label' => t('Send a mail to an arbitrary mail address'),
      '#module' => t('System'),
      '#description' => t('You may use the token replacements for the mail\'s message body, to and from fields as well as for the mail\'s subject.'),
    ),
    'workflow_ng_action_drupal_message' => array(
      '#label' => t('Show a configurable message on the site'),
      //we are not working with defined arguments, instead we make use of all available arguments
      '#arguments' => array(),
      '#module' => t('System'),
    ),
    'workflow_ng_action_drupal_goto' => array(
      '#label' => t('Page redirect'),
      //we are not working with defined arguments, instead we make use of all available arguments
      '#arguments' => array(),
      '#module' => t('System'),
    ),
    'workflow_ng_action_watchdog' => array(
      '#label' => t('Log to watchdog'),
      //we are not working with defined arguments, instead we make use of all available arguments
      '#arguments' => array(),
      '#module' => t('System'),
    ),
  );
}

/**
 * Action: Send mail to all users of a specific role group(s).
 */
function workflow_ng_action_mail_to_users_of_role($settings, &$arguments, &$log) {
  $recipients = $settings['recipients'];
  $names = array('from', 'subject');
  extract( workflow_ng_token_replace_all(array('message' => 'message') + $names, $settings, $arguments, $log) );

  foreach ($names as $key) {
    $$key = str_replace(array("\r", "\n"), '', $$key);
  }

  // All authenticated users, which is everybody.
  if ($recipients[DRUPAL_AUTHENTICATED_RID]) {
    $result = db_query('SELECT mail FROM {users} WHERE uid > 0');
  }
  else {
    $roles = implode(',', $recipients);
    // Avoid sending emails to members of two or more target role groups.
    $result = db_query('SELECT DISTINCT u.mail FROM {users} u INNER JOIN {users_roles} r ON u.uid = r.uid WHERE r.rid IN ('. $roles .')');
  }

  // Now, actually send the mails.
  while ($account = db_fetch_object($result)) {
    if (drupal_mail('workflow-ng-mass-user-mail-by-role', $account->mail, $subject, $message, $from)) {
      watchdog('action', t('Sent email to %recipient', array('%recipient' => $account->mail)), WATCHDOG_NOTICE);
    }
    else {
      watchdog('error', t('Unable to send email to %recipient', array('%recipient' => $account->mail)), WATCHDOG_ERROR);
    }
  }
}

function workflow_ng_action_mail_to_users_of_role_form($settings = array(), $argument_info) {
  // Select only non-anonymous user roles because anonymous users won't have emails.
  $roles = user_roles(TRUE);

  $form['recipients'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Recipient Role Groups'),
    '#prefix' => t('WARNING: This may cause problems if there are too many users of these roles on your site, as your server may not be able to handle all the mail requests all at once.'),
    '#required' => TRUE,
    '#default_value' => $settings['recipients'],
    '#options' => $roles,
    '#description' => t('Select the roles whose users should receive this email.'),
  );
  $form['from'] = array(
    '#type' => 'textfield',
    '#title' => t('Sender'),
    '#default_value' => isset($settings['from']) ? $settings['from'] : variable_get('site_mail', ini_get('sendmail_from')),
    '#description' => t('The mail\'s "from" address.'),
    '#required' => TRUE,
  );
  $form['subject'] = array(
    '#type' => 'textfield',
    '#title' => t('Subject'),
    '#default_value' => $settings['subject'],
    '#description' => t('The mail\'s subject.'),
  );
  $form['message'] = array(
    '#type' => 'textarea',
    '#title' => t('Message'),
    '#default_value' => $settings['message'],
    '#description' => t('The mail\'s message body.'),
  );
  workflow_ng_token_replacement_help($form, $argument_info);
  return $form;
}

function workflow_ng_action_mail_to_users_of_role_submit($form_id, $form_values) {
  $form_values['recipients'] = array_filter($form_values['recipients']);
  return workflow_ng_token_get_settings(array('from', 'subject', 'message'), $form_values) + array('recipients' => $form_values['recipients']);
}

/*
 * Action Implementation: Show a configureable message
 */
function workflow_ng_action_drupal_message($settings, &$arguments, &$log) {
  extract( workflow_ng_token_replace_all(array('message'), $settings, $arguments, $log) );
  drupal_set_message($message, $settings['error'] ? 'error' : 'status');
}

/*
 * Action "Show a configureable message" configuration form
 */
function workflow_ng_action_drupal_message_form($settings = array(), $argument_info) {
  $form = array();
  $form['message'] = array(
    '#type' => 'textarea',
    '#title' => t('Message'),
    '#default_value' => $settings['message'],
    '#description' => t('The message that should be displayed.'),
  );
  workflow_ng_token_replacement_help($form, $argument_info);
  $form['error'] = array(
    '#type' => 'checkbox',
    '#title' => t('Display as error message'),
    '#default_value' => $settings['error'],
  );
  return $form;
}

function workflow_ng_action_drupal_message_submit($form_id, $form_values) {
  //returns the needed settings
  $settings = workflow_ng_token_get_settings(array('message'), $form_values);
  return $settings + array('error' => $form_values['error']);
}

/*
 * Action Implementation: Page redirect
 */
function workflow_ng_action_drupal_goto($settings, &$arguments, &$log) {
  extract( workflow_ng_token_replace_all(array('path', 'query', 'fragment'), $settings, $arguments, $log) );

  if ($settings['force']) {
    if (strpos($query, 'destination') === FALSE) {
      if (!empty($query)) {
        $query .= '&';
      }
      $query .= $_REQUEST['destination'] ? 'destination='. urlencode($_REQUEST['destination']) : NULL;
    }
    unset($_REQUEST['destination']);
  }
  if ($settings['override']) {
    //override the destination parameter, so another drupal_goto goes to our url :)
    $_REQUEST['destination'] = urlencode($path) . ($query ? '?'. urlencode($query) : '').($fragment ? '#'. urlencode($fragment) : '');
  }
  else {
    drupal_goto($path, $query ? $query : NULL, $fragment ? $fragment : NULL);
  }
}

/*
 * Action "Page redirect" configuration form
 */
function workflow_ng_action_drupal_goto_form($settings = array(), $argument_info, $event) {
  $form['redirect'] = array(
    '#type' => 'item',
    '#prefix' => '<div class="container-inline">',
    '#suffix' => '</div>',
    '#title' => t('To'),
    '#description' => '<div style="display:block">'. t('Enter a Drupal path, path alias, or external URL to redirect to. Enter (optional) queries after "?" and (optional) anchor after "#". Most redirects will not contain queries or fragment anchors.') .'</div>',
  );
  $form['redirect']['path'] = array(
    '#type' => 'textfield',
    '#size' => 30,
    '#default_value' => drupal_get_path_alias($settings['path']),
  );
  $form['redirect'][] = array(
    '#value' => '?',
  );
  $form['redirect']['query'] = array(
    '#type' => 'textfield',
    '#size' => 12,
    '#default_value' => $settings['query'],
  );
  $form['redirect'][] = array(
    '#value' => '#',
  );
  $form['redirect']['fragment'] = array(
    '#type' => 'textfield',
    '#size' => 12,
    '#default_value' => $settings['fragment'],
  );
  $form['force'] = array(
    '#type' => 'checkbox',
    '#title' => t('Force redirecting to the given path, even if a destination parameter is given'),
    '#description' => t('Per default drupal doesn\'t redirect to the given path, if a destination parameter is set. '.
      'Instead it redirects to the given destination paramter. Most times, the destination parameter is set by appending it to the URL, e.g. !example_url', array('!example_url' => 'http://example.com/user/login?destination=node/2')),
    '#default_value' => $settings['force'],
    '#weight' => 7,
  );
  $form['override'] = array(
    '#type' => 'checkbox',
    '#title' => t('Override another path redirect by setting the destination parameter'),
    '#description' => t('If checked, the path redirect isn\'t initiated by this action, but the destination parameter gets set. '.
      'So if something else redirects, the redirect will use the configured path.'),
    '#default_value' => $settings['override'],
    '#weight' => 8,
  );
  if (!isset($settings['override'])) {
    //a formAPI bug lets default_value not work, so we have to care about this ourself
    if (isset($event['#redirect']) && $event['#redirect']) {
      $form['override']['#attributes']['checked'] = 'checked';
    }
  }
  workflow_ng_token_replacement_help($form, $argument_info);
  return $form;
}

function workflow_ng_action_drupal_goto_submit($form_id, $form_values) {
  //returns the needed settings
  $settings = workflow_ng_token_get_settings(array('path', 'query', 'fragment'), $form_values);
  return $settings + array('force' => $form_values['force'], 'override' => $form_values['override']);
}

/*
 * Action Implementation: Watchdog
 */
function workflow_ng_action_watchdog($settings, &$arguments, &$log) {
  extract( workflow_ng_token_replace_all(array('message', 'link'), $settings, $arguments, $log) );

  watchdog($settings['type'], $message, $settings['severity'], $link);
}

/*
 * Action "Watchdog" configuration form
 */
function workflow_ng_action_watchdog_form($settings = array(), $argument_info) {
  $form['severity'] = array(
    '#type' => 'select',
    '#title' => t('Severity'),
    '#options' => array(
      WATCHDOG_NOTICE => t('Notice'),
      WATCHDOG_WARNING => t('Warning'),
      WATCHDOG_ERROR => t('Error'),
    ),
    '#default_value' => isset($settings['type']) ? $settings['type'] : WATCHDOG_NOTICE,
    '#required' => TRUE,
  );
  $form['type'] = array(
    '#type' => 'textfield',
    '#title' => t('Category'),
    '#default_value' => isset($settings['type']) ? $settings['type'] : 'workflow-ng',
    '#description' => t('The category to which this message belongs.'),
    '#required' => TRUE,
  );
  $form['message'] = array(
    '#type' => 'textfield',
    '#title' => t('Message'),
    '#default_value' => isset($settings['message']) ? $settings['message'] : '',
    '#description' => t('The message to log. You may make use of the replacements.'),
    '#required' => TRUE,
  );
  $form['link'] = array(
    '#type' => 'textfield',
    '#title' => t('Link (optional)'),
    '#default_value' => isset($settings['link']) ? $settings['link'] : '',
    '#description' => t('A link to associate with the message. You may make use of the replacements.'),
  );
  workflow_ng_token_replacement_help($form, $argument_info);
  return $form;
}

function workflow_ng_action_watchdog_submit($form_id, $form_values) {
  $vars = drupal_map_assoc(array('severity', 'type'));
  $settings = workflow_ng_token_get_settings(array('message', 'link'), $form_values);
  return array_intersect_key($form_values, $vars) + $settings;
}

/*
 * Action Implementation: Send a mail to a user
 */
function workflow_ng_action_mail_to_user($user, $settings, &$arguments, &$log) {
  //we also handle $settings['to'] if it's set
  $names = isset($settings['to']) ? array('from', 'subject', 'to') : array('from', 'subject');
  extract( workflow_ng_token_replace_all(array('message' => 'message') + $names, $settings, $arguments, $log) );

  foreach ($names as $key) {
    $$key = str_replace(array("\r", "\n"), '', $$key);
  }
  $to = isset($to) ? $to : $user->mail;

  if (drupal_mail('workflow_ng_action_mail', $to, $subject, $message, $from)) {
    watchdog('action', t('Sent email to %recipient', array('%recipient' => ($to == $user->mail) ? $user->name : $to)), WATCHDOG_NOTICE, ($to == $user->mail) ? theme('username', $user) : NULL);
  }
  else {
    watchdog('error', t('Unable to send email to %recipient', array('%recipient' => ($to == $user->mail) ? $user->name : $to)), WATCHDOG_NOTICE, ($to == $user->mail) ? theme('username', $user) : NULL);
  }
}

/*
 * Action "Send a mail to a user" configuration form
 */
function workflow_ng_action_mail_to_user_form($settings = array(), $argument_info) {
  $form['from'] = array(
    '#type' => 'textfield',
    '#title' => t('Sender'),
    '#default_value' => isset($settings['from']) ? $settings['from'] : variable_get('site_mail', ini_get('sendmail_from')),
    '#description' => t('The mail\'s "from" address.'),
    '#required' => TRUE,
  );
  $form['subject'] = array(
    '#type' => 'textfield',
    '#title' => t('Subject'),
    '#default_value' => $settings['subject'],
    '#description' => t('The mail\'s subject.'),
  );
  $form['message'] = array(
    '#type' => 'textarea',
    '#title' => t('Message'),
    '#default_value' => $settings['message'],
    '#description' => t('The mail\'s message body.'),
  );
  workflow_ng_token_replacement_help($form, $argument_info);
  return $form;
}

function workflow_ng_action_mail_to_user_submit($form_id, $form_values) {
  return workflow_ng_token_get_settings(array('from', 'subject', 'message'), $form_values);
}

/*
 * Action Implementation: workflow_ng_action_mail
 * This action makes use of the workflow_ng_action_mail_to_user action implementation
 */
function workflow_ng_action_mail($settings, &$arguments, &$log) {
  workflow_ng_action_mail_to_user(NULL, $settings, $arguments, $log);
}

/*
 * Action "Send a mail to an arbitrary mail address" configuration form
 */
function workflow_ng_action_mail_form($settings = array(), $argument_info) {
  $form = workflow_ng_action_mail_to_user_form($settings, $argument_info);
  $form['to'] = array(
    '#type' => 'textfield',
    '#title' => t('Recipient'),
    '#default_value' => $settings['to'],
    '#description' => t('The mail\'s recipient address. You may separate multiple addresses with ",".'),
    '#required' => TRUE,
    '#weight' => -1,
  );
  return $form;
}

function workflow_ng_action_mail_submit($form_id, $form_values) {
  return workflow_ng_token_get_settings(array('to', 'from', 'subject', 'message'), $form_values);
}